/*
 * File:		I2C.c
 * Purpose:		simplified I2C driver:
 *                      master init and transfer functions
 *
 * Notes:		
 *  
 */
 
#include "i2c.h"

/********************************************************************/
/* Initializes the I2C in master mode.
*/
void i2c_init(I2C_Type * module)
{
    /* Enable the clock for the I2C module */
    if (module == I2C0)
      CCM->CCGR4 |= CCM_CCGR4_CG6_MASK;
    if (module == I2C1)
      CCM->CCGR4 |= CCM_CCGR4_CG7_MASK;
    if (module == I2C2)
      CCM->CCGR4 |= CCM_CCGR4_CG8_MASK;
    if (module == I2C3)
      CCM->CCGR4 |= CCM_CCGR4_CG9_MASK;
    
    /* Set the divider for the SCL frequency */
    module->IBFD = 0x38;
    
    /* Set the slave address - won't actually be used */
    module->IBAD = 0xC0;
    
    /* Clear MDIS to enable I2C and enable interrupts */
    module->IBCR = I2C_IBCR_IBIE_MASK;
      
}  
/********************************************************************/
void i2c0_pin_init(void)
{
  
#if (defined(TOWER))  
    
  /* Mux input */
  IOMUXC->I2C0_IPP_SDA_IND_SELECT_INPUT = 0x01;
  IOMUXC->I2C0_IPP_SCL_IND_SELECT_INPUT = 0x01;
  
  IOMUXC->SINGLE.PTB14 = IOMUXC_PTA6_MUX_MODE(2) 
                          | IOMUXC_PTA6_SPEED(0)
                          | IOMUXC_PTA6_ODE_MASK 
                          | IOMUXC_PTA6_DSE(4)  
                          | IOMUXC_PTA6_IBE_MASK
                          | IOMUXC_PTA6_OBE_MASK;
  IOMUXC->SINGLE.PTB15 = IOMUXC_PTA6_MUX_MODE(2) 
                          | IOMUXC_PTA6_SPEED(0)
                          | IOMUXC_PTA6_ODE_MASK  
                          | IOMUXC_PTA6_DSE(4)  
                          | IOMUXC_PTA6_IBE_MASK
                          | IOMUXC_PTA6_OBE_MASK;
  
#else
  #error "No valid hardware platform defined."  
#endif  
}  
/********************************************************************/
void i2c_reg_write(I2C_Type * module, uint8 slave_addr, uint8 reg_addr, uint8 data)
{
    /* Configure the I2C for master transmit */
    module->IBCR |= I2C_IBCR_MSSL_MASK | I2C_IBCR_TXRX_MASK;
    
    /* Write the slave address to the data register to transmit */
    module->IBDR = slave_addr;
    
    /* Wait for the transfer to complete */
    i2c_wait();
    
    /* Write the register address to the data register to transmit */
    module->IBDR = reg_addr;

    /* Wait for the transfer to complete */
    i2c_wait();

    /* Write the slave address to the data register to transmit */
    module->IBDR = data;
    
    /* Wait for the transfer to complete */
    i2c_wait();

    /* Transfer complete. Clear MSSL bit to send a STOP */
    module->IBCR = I2C_IBCR_IBIE_MASK;
}  
/********************************************************************/
uint8 i2c_reg_read(I2C_Type * module, uint8 slave_addr, uint8 reg_addr)
{
    uint8 temp;
    
    /* Configure the I2C for master transmit */
    module->IBCR |= I2C_IBCR_MSSL_MASK | I2C_IBCR_TXRX_MASK;
    
    /* Send write slave address */
    module->IBDR = slave_addr & ~0x1;
    
    /* Wait for the transfer to complete */
    i2c_wait();

    /* Write the register address to the data register to transmit */
    module->IBDR = reg_addr;

    /* Wait for the transfer to complete */
    i2c_wait();

    /* Do a repeated start */
    module->IBCR |= I2C_IBCR_RSTA_MASK;

    /* Send Slave Address for a read */
    module->IBDR = slave_addr;

    /* Wait for the transfer to complete */
    i2c_wait();

    /* Configure the I2C to receive */
    module->IBCR &= ~I2C_IBCR_TXRX_MASK;

    /* Read a dummy byte from the data register */
    temp = module->IBDR;

    /* Wait for the transfer to complete */
    i2c_wait();

    /* Transfer complete. Clear MSSL bit to send a STOP */
    module->IBCR = I2C_IBCR_IBIE_MASK;
    
    /* Read data register again to get actual returned value */
    return module->IBDR;
}  
/********************************************************************/
void i2c_wait(void)
{
  uint32 i;
  
  for( i=0; i<0x10000; i++);
  
  /* Wait for the TCF flag to clear */
//  while( (I2C0->IBSR & I2C_IBSR_TCF_MASK));  
  
  /* Wait for the transfer to complete */
//  while( !(I2C0->IBSR & I2C_IBSR_TCF_MASK));
  
}  
/********************************************************************/



